"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = void 0;
const consul_1 = __importDefault(require("consul"));
const fs_1 = require("fs");
const util_1 = require("util");
const js_yaml_1 = __importDefault(require("js-yaml"));
const readFileAsync = (0, util_1.promisify)(fs_1.readFile);
function parseString(str) {
    if (str === "true")
        return true;
    if (str === "false")
        return false;
    if (str === "null")
        return null;
    if (str === "undefined")
        return undefined;
    const num = Number(str);
    if (!isNaN(num))
        return num;
    if (str[0] === "{")
        return JSON.parse(str);
    return str;
}
async function getConsulConfigNested(prefix) {
    if (!prefix)
        return undefined;
    const opts = { promisify: true };
    if (process.env.CONSUL_HOST) {
        opts.host = process.env.CONSUL_HOST;
    }
    if (process.env.CONSUL_PORT) {
        opts.port = process.env.CONSUL_PORT;
    }
    console.info("Fetching config from Consul host=%s port=%s prefix=%s", opts.host || "default", opts.port || "default", prefix);
    const consulClient = (0, consul_1.default)(opts);
    const keys = await consulClient.kv.keys(prefix);
    const values = (await Promise.all(keys.map((key) => consulClient.kv.get(key))));
    return values.reduce((obj, value, i) => {
        const { Key, Value } = value;
        const fullPath = Key.replace(prefix, "");
        const path = fullPath.split("/").filter((s) => s !== "");
        if (!path.length)
            return obj;
        const basePath = path.splice(0, path.length - 1);
        const [fieldName] = path;
        const base = basePath.reduce((x, k) => {
            if (!x[k]) {
                x[k] = {};
            }
            return x[k];
        }, obj);
        if (Value === null) {
            base[fieldName] = {};
        }
        else {
            base[fieldName] = parseString(Value);
        }
        return obj;
    }, {});
}
async function getConsulConfigJson(key) {
    if (!key)
        return undefined;
    const opts = { promisify: true };
    if (process.env.CONSUL_HOST) {
        opts.host = process.env.CONSUL_HOST;
    }
    if (process.env.CONSUL_PORT) {
        opts.port = process.env.CONSUL_PORT;
    }
    console.info("Fetching config from Consul host=%s port=%s key=%s", opts.host || "default", opts.port || "default", key);
    const consulClient = (0, consul_1.default)(opts);
    const result = await consulClient.kv.get(key);
    return JSON.parse(result.Value);
}
async function getConfig(configVars) {
    var _a, _b, _c, _d, _e;
    const path = process.env.CONFIG_PATH || (configVars === null || configVars === void 0 ? void 0 : configVars.configPath);
    const fileConfig = path
        ? js_yaml_1.default.load(await readFileAsync(path, "utf8"))
        : undefined;
    const consulConfigNested = await getConsulConfigNested(process.env.CONSUL_KV_PREFIX);
    const consulConfigJson = await getConsulConfigJson(process.env.CONSUL_KV_KEY);
    const mergedConfig = Object.assign({}, fileConfig, consulConfigNested, consulConfigJson, configVars);
    const mqttUri = process.env.MQTT_URI || ((_a = mergedConfig.mqtt) === null || _a === void 0 ? void 0 : _a.uri);
    if (!mqttUri) {
        throw new Error("MQTT URI Required.");
    }
    const mqttPrefix = process.env.MQTT_PREFIX || ((_b = mergedConfig.mqtt) === null || _b === void 0 ? void 0 : _b.prefix) || "";
    const mqttSubscriptions = ((_c = process.env.MQTT_SUBSCRIPTIONS) === null || _c === void 0 ? void 0 : _c.split(",")) ||
        ((_d = mergedConfig.mqtt) === null || _d === void 0 ? void 0 : _d.subscriptions) ||
        [];
    const mqtt = {
        uri: mqttUri,
        prefix: mqttPrefix,
        subscriptions: mqttSubscriptions,
    };
    const httpPort = (process.env.HTTP_PORT && parseInt(process.env.HTTP_PORT)) ||
        ((_e = mergedConfig.http) === null || _e === void 0 ? void 0 : _e.port) ||
        null;
    const http = {
        port: httpPort,
    };
    const service = (process.env.SERVICE_CONFIG && JSON.parse(process.env.SERVICE_CONFIG)) ||
        mergedConfig.service ||
        {};
    return {
        mqtt,
        http,
        service,
    };
}
exports.getConfig = getConfig;
//# sourceMappingURL=config.js.map